--[[============================================================================
qbasic_levels.lua
============================================================================]]--
-- QBasic Nibbles levels for renoise Nibbles

-- Get access to tool globals
local nibbles = get_nibbles()

local function qbasic_level_defaults()
    --[[
    Set level defaults for QBasic levels
    --]]
    
    -- Matrix size of current level:
    -- TODO make this actually work
    -- in order to implement freely sized levels
    nibbles.level_defaults.field_size = {
        x = 80,
        y = 50,
    }

    -- The area where food is allowed to appear in current level
    nibbles.level_defaults.food_area = {
        --minimum x to have food appear
        x_min = 1,
        --maximum x to have food appear
        x_max = 79,
        --minimym y to have food appear
        y_min = 4,
        --maximum y to have food appear
        y_max = 49,
    }
end

--------------------------------------------------------------------------------
-- QBasic Levels: Default map setup
--------------------------------------------------------------------------------

local function qbasic_level_walls()
    --[[
    Draw the walls that are present in all the QBasic levels
    --]]
    for col = 1, 80 do
        set_cell_color(col, 3, nibbles.color_wall)
        set_cell_color(col, 50, nibbles.color_wall)
    end
    for row = 4, 49 do
        set_cell_color(1, row, nibbles.color_wall)
        set_cell_color(80, row, nibbles.color_wall)
    end
    return true
end

--------------------------------------------------------------------------------
-- QBasic Levels
--------------------------------------------------------------------------------

function qbasic_level_1()
    -- Nibbles QBasic Level 1
    qbasic_level_defaults()
    clear_cells()
    snake_start(50, 25, "right")
    qbasic_level_walls()
    -- Draw level
    -- QBasic Level 1 is empty
end

function qbasic_level_2()
    -- Nibbles QBasic Level 2
    qbasic_level_defaults()
    clear_cells()
    snake_start(60, 7, "right")
    qbasic_level_walls()
    -- Draw level
    for i = 20, 60 do
        set_cell_color(i, 25, nibbles.color_wall)     
    end
end

function qbasic_level_3()
    -- Nibbles QBasic Level 3
    qbasic_level_defaults()
    clear_cells()
    snake_start(50, 25, "up")
    qbasic_level_walls()
    -- Draw level
    for i = 10, 40 do
        set_cell_color(20, i, nibbles.color_wall)
        set_cell_color(60, i, nibbles.color_wall)
    end
end

function qbasic_level_4()
    -- Nibbles QBasic Level 4
    qbasic_level_defaults()
    clear_cells()
    snake_start(60, 7, "left")
    qbasic_level_walls()
    -- Draw level
    for i = 4, 30 do
        set_cell_color(20, i, nibbles.color_wall)
        set_cell_color(60, 53 - i, nibbles.color_wall)
    end
    for i = 2, 40 do
        set_cell_color(i, 38, nibbles.color_wall)
        set_cell_color(81 - i, 15, nibbles.color_wall)
    end
end

function qbasic_level_5()
    -- Nibbles QBasic Level 5
    qbasic_level_defaults()
    clear_cells()
    snake_start(50, 25, "up")
    qbasic_level_walls()
    -- Draw level
    for i = 13, 39 do
        set_cell_color(21, i, nibbles.color_wall)
        set_cell_color(59, i, nibbles.color_wall)
    end
    for i = 23, 57 do
        set_cell_color(i, 11, nibbles.color_wall)
        set_cell_color(i, 41, nibbles.color_wall)
    end
end

function qbasic_level_6()
    -- Nibbles QBasic Level 6
    qbasic_level_defaults()
    clear_cells()
    snake_start(65, 7, "down")
    qbasic_level_walls()
    -- Draw level
    for i = 4, 49 do
        if i > 30 or i < 23 then
            set_cell_color(10, i, nibbles.color_wall)      
            set_cell_color(20, i, nibbles.color_wall)      
            set_cell_color(30, i, nibbles.color_wall)      
            set_cell_color(40, i, nibbles.color_wall)      
            set_cell_color(50, i, nibbles.color_wall)      
            set_cell_color(60, i, nibbles.color_wall)      
            set_cell_color(70, i, nibbles.color_wall)      
        end
    end
end

function qbasic_level_7()
    -- Nibbles QBasic Level 7
    qbasic_level_defaults()
    clear_cells()
    snake_start(65, 7, "down")
    qbasic_level_walls()
    -- Draw level
    for i = 4, 49, 2 do
        set_cell_color(40, i, nibbles.color_wall)
    end
end

function qbasic_level_8()
    -- Nibbles QBasic Level 8
    qbasic_level_defaults()
    clear_cells()
    snake_start(65, 7, "down")
    qbasic_level_walls()
    -- Draw level
    for i = 4, 40 do
        set_cell_color(10, i, nibbles.color_wall)
        set_cell_color(20, 53 - i, nibbles.color_wall)
        set_cell_color(30, i, nibbles.color_wall)
        set_cell_color(40, 53 - i, nibbles.color_wall)
        set_cell_color(50, i, nibbles.color_wall)
        set_cell_color(60, 53 - i, nibbles.color_wall)
        set_cell_color(70, i, nibbles.color_wall)
    end
end

function qbasic_level_9()
    -- Nibbles QBasic Level 9
    qbasic_level_defaults()
    clear_cells()
    snake_start(75, 40, "up")
    qbasic_level_walls()
    -- Draw level
    for i = 6, 47 do
        set_cell_color(i, i, nibbles.color_wall)  
        set_cell_color(i + 28, i, nibbles.color_wall)  
    end
end

function qbasic_level_10()
    -- Nibbles QBasic Level 10
    qbasic_level_defaults()
    clear_cells()
    snake_start(65, 7, "down")
    qbasic_level_walls()
    -- Draw level
    for i = 4, 49, 2 do
        set_cell_color(10, i, nibbles.color_wall)
        set_cell_color(20, i + 1, nibbles.color_wall)
        set_cell_color(30, i, nibbles.color_wall)
        set_cell_color(40, i + 1, nibbles.color_wall)
        set_cell_color(50, i, nibbles.color_wall)
        set_cell_color(60, i + 1, nibbles.color_wall)
        set_cell_color(70, i, nibbles.color_wall)
    end
end
